import java.util.ArrayList;
import java.util.List;

public class Equation {
    private String sens;
    private int nbContraintes;
    private List<Double> funcObj;
    private List<Double> rightVec;
    private Matrix mat;


    /**
     * getter sens
     *
     * @return min | max | null
     */
    public String getSens() {
        return sens;
    }

    /**
     * Setter sens
     *
     * @param sens max | min
     */
    public void setSens(String sens) {
        // prends en compte majuscule et minuscule
        switch (sens.toLowerCase()) {
            // Java version 14 minimum
            case "min":
            case "max":
                this.sens = sens.toLowerCase();
                break;
            default:
                this.sens = null;
                System.err.println("Le sens ne peut que être min ou max");
                System.exit(1);
        }
    }

    public int getNbContraintes() {
        if (this.nbContraintes != -1)
            return this.nbContraintes;
        else {
            System.err.println("Une erreur a été détectée");
            return -1;
        }
    }

    public List<Double> getFuncObj() {
        return funcObj;
    }

    public void setFuncObj(double d) {
        this.funcObj.add(d);
    }

    public List<Double> getRightVec() {
        return rightVec;
    }

    public void setRightVec(double d) {
        this.rightVec.add(d);
    }

    public double getMatAtId(int w, int h) {
        return mat.getData(w, h);
    }

    public Matrix getMat() {
        return this.mat;
    }

    /**
     * Constructeur
     *
     * @param w hauteur de la matrice
     * @param h largeur de la matrice
     */
    public Equation(int w, int h) {
        this.sens = null;
        this.funcObj = new ArrayList<>();
        this.rightVec = new ArrayList<>();
        this.nbContraintes = h;
        mat = new Matrix(w, h);
    }

    /**
     * Récupérer les informations de la première ligne (sens + fonction obj)
     *
     * @param elements inéquation
     */
    public void setFirstLine(String[] elements) throws NumberFormatException{
        setSens(elements[0]);
        if (this.sens.equals("min")) {
            for (int i = 1; i < elements.length; i++) {
                this.funcObj.add(Double.parseDouble(elements[i]));
            }
        } else {
            for (int i = 1; i < elements.length; i++) {
                this.funcObj.add(-Double.parseDouble(elements[i]));
            }
        }
    }

    /**
     * Créer le vecteur éléments de droite et la matrice Amxn quand le sens est à min
     *
     * @param elements les elements
     * @param line     la line en question
     */
    public int createEq(String[] elements, int line) throws NumberFormatException {
        // vecteur de droite
        switch (elements[getNbContraintes()]) {
            case "=": // Si ax=b => ax <= b; ax >= b
                double res = Double.parseDouble(elements[elements.length - 1]);
                // Vecteur de droite
                this.rightVec.add(res);

                // Si 0.0, ne pas écrire -0.0
                if (res == 0.0) this.rightVec.add(res);
                else this.rightVec.add(-res);

                // Matrice
                this.mat.matrixRealloc(this.mat.getX() + 1, this.mat.getY());
                for (int i = 0; i < this.nbContraintes; i++) {
                    double tmp = Double.parseDouble(elements[i]);
                    this.mat.setData(line, i, tmp);
                    // Si 0.0, ne pas écrire -0.0
                    if (tmp == 0.0) this.mat.setData(line + 1, i, tmp);
                    else this.mat.setData(line + 1, i, -tmp);
                }
                // On rajoute une ligne
                line += 1;
                break;
            case ">=":
                // Vecteur de droite
                this.rightVec.add(-Double.parseDouble(elements[elements.length - 1]));

                // Matrice
                for (int i = 0; i < getNbContraintes(); i++)
                    this.mat.setData(line, i, -Double.parseDouble(elements[i]));
                break;
            case "<=":
                // Vecteur de droite
                this.rightVec.add(Double.parseDouble(elements[elements.length - 1]));

                // Matrice
                for (int i = 0; i < getNbContraintes(); i++)
                    this.mat.setData(line, i, Double.parseDouble(elements[i]));
                break;
            default:
                System.err.println("Le nombre d'arguments dans l'inéquation est incorrecte.");
                System.exit(1);
        }
        return line;
    }

    /**
     * Print les vecteurs et la matrice
     */
    public void printEq() {
        // Fonction obj
        System.out.println("Fonction obj: " + getFuncObj());

        // Vecteur membre de droite
        System.out.println("Vecteur membre de droite: " + getRightVec());

        System.out.println("Matrice Amxn:");
        for (int i = 0; i < this.mat.getX(); i++) {
            for (int j = 0; j < this.mat.getY(); j++) {
                if (this.mat.getData(i, j) < 0.0)
                    System.out.print(this.mat.getData(i, j) + " ");
                else
                    System.out.print(" " + this.mat.getData(i, j) + " ");
            }
            System.out.println();
        }
    }
}
